# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

""" ContactView emulator for Addressbook App tests """

import logging

from autopilot import logging as autopilot_logging
from autopilot.introspection.dbus import StateNotFoundError
from ubuntuuitoolkit import emulators as uitk

from address_book_app.emulators import get_nearest_parent_matching_type

logger = logging.getLogger(__name__)


class ContactView(uitk.UbuntuUIToolkitEmulatorBase):
    """ContactView emulator class"""

    def __init__(self, *args):
        super(ContactView, self).__init__(*args)

    @autopilot_logging.log_action(logger.info)
    def click_button(self, objectname):
        """Press a button that matches objectname.

        :param objectname: The name of the object.
        :raises StateNotFoundError: If the matching object is not found.

        """
        buttons = self.select_single("Button",
                                     objectName=objectname,
                                     visible=True)
        self.pointing_device.click_object(button)

    @autopilot_logging.log_action(logger.info)
    def cancel(self):
        """Press the cancel button displayed when pick mode is enabled."""
        self.click_button("DialogButtons.rejectButton")

    @autopilot_logging.log_action(logger.info)
    def delete(self):
        """Press the delete button displayed when pick mode is enabled."""
        self.click_button("DialogButtons.acceptButton")
        self.get_contacts()

    def get_contact_detail_phone_number_view(self, phone_number):
        """Returns the View associated with the given phone number.

        :param phone_number: The number listed for contact, e.g. '8675309'.
        :return: The matching view.
        :raises StateNotFoundError: If the phone_number is not found.

        """
        phone_number_label = self.select_single('Label',
                                                text=phone_number)
        return get_nearest_parent_matching_type(
            phone_number_label,
            'ContactDetailPhoneNumberView')

    def _star_is_illuminated_for_phone_number(self, phone_number):
        """Is the star associated with the given phone number illuminated?

        :param phone_number: The number listed for contact, e.g. '8675309'.
        :return: A Boolean.
        :raises StateNotFoundError: If the phone_number is not found.

        """
        contact_detail = self.get_contact_detail_phone_number_view(
            phone_number)
        return contact_detail.iconSource == 'artwork:/favorite-selected.svg'

    @autopilot_logging.log_action(logger.info)
    def click_phone_number(self, phone_number):
        """Tap/click the label with text matching the given phone number.

        :param phone_number: The number listed for contact, e.g. '8675309'.
        :raises StateNotFoundError: If the phone_number is not found.

        """
        phone_number_label = self.wait_select_single('Label',
                                                     text=phone_number)
        self.pointing_device.click_object(phone_number_label)

    def phone_number_is_favorite(self, phone_number):
        """Is the given phone number a favorite?

        :param phone_number: A phone number, e.g. '8675309'.
        :raises StateNotFoundError: If the phone number is not found.
        :returns: Boolean.

        """
        return self._star_is_illuminated_for_phone_number(phone_number)
