# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

"""Tests of favoriting contacts for the Addressbook App"""

from __future__ import absolute_import

from autopilot.introspection.dbus import StateNotFoundError

from address_book_app.tests import AddressBookAppTestCase
from address_book_app.emulators import get_nearest_parent_matching_type
from address_book_app.emulators.contact_list_page import ContactListPage
from address_book_app.emulators.contact_view import ContactView
from address_book_app.emulators.address_book import AddressBook


class TestFavorite(AddressBookAppTestCase):
    """Test for favoriting contacts"""

    def setUp(self):
        AddressBookAppTestCase.QTCONTACTS_MANAGER_OVERRIDE_MEMORY = False
        super(TestFavorite, self).setUp()
        self.address_book = AddressBook()

    def test_add_favorite(self):
        self.address_book.create_contact(
            name='de Tal;Fulano',
            tel_home='3321 2300')
        contact_list_page = self.main_window.get_contact_list_page()
        contact_list_page.open_contact_view_by_contact_name('Fulano de Tal')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('3321 2300')
        self.assertTrue(
            contact_view.phone_number_is_favorite('3321 2300'))
        self.main_window.go_back()
        self.assertTrue(contact_list_page.get_favorite_by_name('Fulano de Tal'))

    def test_save_multiple_favorites(self):
        self.address_book.create_contact(
            name=';Andrew',
            tel_home='3321 2300')
        self.address_book.create_contact(
            name='Whittard;Beth',
            tel_home='8675309')
        contact_list_page = self.main_window.get_contact_list_page()
        contact_list_page.click_contact_by_name('Beth Whittard')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('8675309')
        self.main_window.go_back()
        self.assertIsNotNone(contact_list_page.get_favorite_by_name('Beth Whittard'))
        contact_list_page.click_contact_by_name('Andrew')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('3321 2300')
        self.main_window.go_back()
        self.assertIsNotNone(contact_list_page.get_favorite_by_name('Andrew'))

    def test_switch_between_favorites(self):
        self.address_book.create_contact(
            name='Whittard;Beth',
            tel_home='8675309',
            tel_work='5551212')
        contact_list_page = self.main_window.get_contact_list_page()
        contact_list_page.click_contact_by_name('Beth Whittard')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('8675309')
        self.assertTrue(contact_view.phone_number_is_favorite('8675309'))
        self.assertFalse(contact_view.phone_number_is_favorite('5551212'))
        contact_view.click_phone_number('5551212')
        self.assertFalse(contact_view.phone_number_is_favorite('8675309'))
        self.assertTrue(contact_view.phone_number_is_favorite('5551212'))
        self.main_window.go_back()
        contact_list_page.click_contact_by_name('Beth Whittard')
        contact_view = self.main_window.get_contact_view_page()
        # NOTE phones not visible from contact list favorites view
        self.assertTrue(contact_view.phone_number_is_favorite('5551212'))
        self.assertFalse(contact_view.phone_number_is_favorite('8675309'))

    def test_remove_favorite(self):
        self.address_book.create_contact(
            name='Whittard;Beth',
            tel_home='8675309')
        contact_list_page = self.main_window.get_contact_list_page()
        contact_list_page.click_contact_by_name('Beth Whittard')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('8675309')
        self.assertTrue(contact_view.phone_number_is_favorite('8675309'))
        self.main_window.go_back()
        contact_list_page.click_contact_by_name('Beth Whittard')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('8675309')
        self.assertFalse(contact_view.phone_number_is_favorite('8675309'))
        self.main_window.go_back()
        self.assertRaises(
            StateNotFoundError,
            lambda: contact_list_page.get_favorite_by_name('Beth Wittard'))
        # FIXME: this passes but appears broken ATM--screen refresh?

    def test_delete_favorite(self):
        self.address_book.create_contact(
            name='Whittard;Beth',
            tel_home='8675309')
        contact_list_page = self.main_window.get_contact_list_page()
        contact_list_page.click_contact_by_name('Beth Whittard')
        contact_view = self.main_window.get_contact_view_page()
        contact_view.click_phone_number('8675309')
        self.assertTrue(contact_view.phone_number_is_favorite('8675309'))
        self.main_window.go_back()
        contact_list_page.click_favorite_by_name('Beth Whittard')
        toolbar = self.main_window.open_toolbar()
        toolbar.click_button('delete')
        self.main_window.go_back()
        self.assertRaises(
            StateNotFoundError,
            lambda: contact_list_page.get_favorite_by_name('Beth Wittard'))
        self.assertRaises(
            StateNotFoundError,
            lambda: contact_list_page.get_contact_by_name('Beth Wittard'))
